==> opengtl_0.9.5.orig.tar.gz: OpenGTL-0.9.5/OpenGTL/GTLCore/AST/CoumpoundExpression.cpp <==
/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "CoumpoundExpression.h"

#include <llvm/Constants.h>
#include <llvm/DerivedTypes.h>
#include <llvm/GlobalVariable.h>
#include <llvm/Instructions.h>

#include "../CodeGenerator_p.h"
#include "../Debug.h"
#include "../Type.h"
#include "../Type_p.h"
#include "../ExpressionResult_p.h"

using namespace GTLCore::AST;

CoumpoundExpression::CoumpoundExpression( const GTLCore::Type* _type, const std::vector<Expression*>& _expressions) : m_expressions(_expressions), m_type(_type)
{
  GTL_ASSERT(_type);
}

CoumpoundExpression::~CoumpoundExpression( )
{
  for( std::vector<Expression*>::iterator it = m_expressions.begin();
       it != m_expressions.end(); ++it)
  {
    delete *it;
  }
}

const GTLCore::Type* CoumpoundExpression::type() const
{
  return m_type;
}

bool CoumpoundExpression::isConstant() const
{
  return m_constant;
}

GTLCore::ExpressionResult CoumpoundExpression::generateValue( GenerationContext& _gc, llvm::BasicBlock* _bb ) const
{
  GTL_DEBUG( m_type->dataType() << " " << Type::ARRAY << " " << Type::STRUCTURE );
  if( m_type->dataType() == Type::ARRAY or m_type->dataType() == Type::VECTOR)
  {
    const GTLCore::Type* arrayType = m_type->embeddedType();
    std::vector< llvm::Constant* > members;
    for(  std::vector<Expression*>::const_iterator it = m_expressions.begin();
       it != m_expressions.end(); ++it)
    {
      members.push_back( _gc.codeGenerator()->convertConstantTo( (*it)->generateValue( _gc, _bb).constant(), (*it)->type(), arrayType ) );
    }
    if( m_type->dataType() == Type::ARRAY )
    {
      std::vector<llvm::Constant*> arrayStruct;
      arrayStruct.push_back( _gc.codeGenerator()->integerToConstant( members.size() ) );
      llvm::Constant* constant = llvm::ConstantArray::get(
                    llvm::ArrayType::get( arrayType->d->type(), members.size()), members );
      llvm::GlobalVariable* gvar = new llvm::GlobalVariable( constant->getType(), true, llvm::GlobalValue::ExternalLinkage, constant, "", _gc.llvmModule() );
      
      llvm::Constant* zero = llvm::ConstantInt::get(llvm::Type::Int32Ty, 0); // Access the structure of the array
      llvm::Constant *Ops[] = {zero, zero };
      llvm::Constant* iptr = llvm::ConstantExpr::getGetElementPtr( gvar, Ops, 2);
      
      arrayStruct.push_back( iptr );
      
      const llvm::StructType* structType = dynamic_cast<const llvm::StructType*>( m_type->d->type() );
      GTL_ASSERT( structType );
      
      return GTLCore::ExpressionResult(
            llvm::ConstantStruct::get( structType, arrayStruct ), type() );
    } else if( m_type->dataType() == Type::VECTOR )
    {
      const llvm::VectorType* vectorType = dynamic_cast<const llvm::VectorType*>( m_type->d->type() );
      GTL_ASSERT( vectorType );
      llvm::Constant* constant = llvm::ConstantVector::get( vectorType, members );
      GTL_DEBUG( "Coumpound constant = " << *constant << " type = " << *m_type );
      return GTLCore::ExpressionResult( constant, m_type );
    }
  } else if( m_type->dataType() == Type::STRUCTURE )
  {
    std::vector<llvm::Constant*> members;
    for( uint i = 0; i < m_expressions.size(); ++i)
    {
      members.push_back( _gc.codeGenerator()->convertConstantTo( m_expressions[i]->generateValue( _gc, _bb).constant(), m_expressions[i]->type(), (*m_type->structDataMembers())[i].type() ) );
    }
    const llvm::StructType* structType = dynamic_cast<const llvm::StructType*>( m_type->d->type() );
    return GTLCore::ExpressionResult(
          llvm::ConstantStruct::get( structType, members ), type() );
  }
  GTL_ABORT("Can't generate coumpound value, neither array, neither vector.");
  return GTLCore::ExpressionResult();
}

void CoumpoundExpression::markAsReturnExpression()
{
  GTL_ABORT("CoumpoundExpression can't be in a return statement");
}

==> opengtl_0.9.9.orig.tar.gz: OpenGTL-0.9.9/OpenGTL/GTLCore/AST/CoumpoundExpression.cpp <==
/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "CoumpoundExpression.h"

#include <llvm/Constants.h>
#include <llvm/DerivedTypes.h>
#include <llvm/GlobalVariable.h>
#include <llvm/Instructions.h>

#include "../CodeGenerator_p.h"
#include "../Debug.h"
#include "../Type.h"
#include "../Type_p.h"
#include "../ExpressionResult_p.h"

using namespace GTLCore::AST;

CoumpoundExpression::CoumpoundExpression( const GTLCore::Type* _type, const std::vector<Expression*>& _expressions) : m_expressions(_expressions), m_type(_type)
{
  GTL_ASSERT(_type);
}

CoumpoundExpression::~CoumpoundExpression( )
{
  for( std::vector<Expression*>::iterator it = m_expressions.begin();
       it != m_expressions.end(); ++it)
  {
    delete *it;
  }
}

const GTLCore::Type* CoumpoundExpression::type() const
{
  return m_type;
}

bool CoumpoundExpression::isConstant() const
{
  return m_constant;
}

GTLCore::ExpressionResult CoumpoundExpression::generateValue( GenerationContext& _gc, llvm::BasicBlock* _bb ) const
{
  GTL_ASSERT( _bb );
  GTL_DEBUG( *m_type << " " << m_type->dataType() << " " << Type::ARRAY << " " << Type::STRUCTURE << " " << Type::VECTOR );
  
  switch( m_type->dataType() )
  {
    case Type::VECTOR:
    {
      GTL_DEBUG( m_expressions.size() << " " << m_type->vectorSize() );
      GTL_ASSERT( m_expressions.size() == m_type->vectorSize() );
      llvm::Value* resultLoad = CodeGenerator::createVector( m_type );
      // Affect the same value to each element of the vector
      for(std::size_t i = 0; i < m_expressions.size(); ++i)
      {
        llvm::Value* val = m_expressions[i]->generateValue( _gc, _bb ).value();
        GTL_DEBUG( i << " " << *val );
        resultLoad = llvm::InsertElementInst::Create( resultLoad, val, i, "", _bb );
      }
      return GTLCore::ExpressionResult( resultLoad, m_type );
    }
      break;
    case Type::ARRAY:
      GTL_ABORT("Unimplemented");
    case Type::STRUCTURE:
      GTL_ABORT("Unimplemented");
    default:
      GTL_ABORT("Unsupported");
  }
}

void CoumpoundExpression::markAsReturnExpression()
{
  for(std::size_t i = 0; i < m_expressions.size(); ++i)
  {
    m_expressions[i]->markAsReturnExpression();
  }
}
